--Chromatic Aberration effect filter v1.1 by khaver

obs = obslua
bit = require("bit")

SETTING_RAD = 'radjust'
SETTING_GAD = 'gadjust'
SETTING_BAD = 'badjust'
SETTING_CENT = 'center'
SETTING_SHOW = 'show'

TEXT_RAD = 'Red adjust'
TEXT_GAD = 'Green adjust'
TEXT_BAD = 'Blue adjust'
TEXT_CENT = 'Protect center'
TEXT_SHOW = 'Show protected center'

source_info = {}
source_info.id = 'filter-chomatic'
source_info.type = obs.OBS_SOURCE_TYPE_FILTER
source_info.output_flags = bit.bor(obs.OBS_SOURCE_VIDEO, obs.OBS_SOURCE_SRGB)

-- Returns the description displayed in the Scripts window
function script_description()
  return [[Filter for adding chromatic aberration by khaver]]
end

function script_load(settings)
  obs.obs_register_source(source_info)
end

function set_render_size(filter)
    target = obs.obs_filter_get_target(filter.source)

    local width, height
    if target == nil then
        width = 0
        height = 0
    else
        width = obs.obs_source_get_base_width(target)
        height = obs.obs_source_get_base_height(target)
    end

    filter.width = width
    filter.height = height
end

source_info.get_name = function()
    return "Chromatic Aberration"
end

source_info.create = function(settings, source)

    local filter = {}
    filter.source = source
	--filter.width = 1
	--filter.height = 1

    set_render_size(filter)

    obs.obs_enter_graphics()
    --local effect_path = script_path() .. 'filter-chromatic.effect'
    --filter.effect = obs.gs_effect_create_from_file(effect_path, nil)
    filter.effect = obs.gs_effect_create(shader, nil, nil)
    obs.obs_leave_graphics()

    if filter.effect ~= nil then
		filter.params = {}
        filter.params.radjust = obs.gs_effect_get_param_by_name(filter.effect, 'radjust')
        filter.params.gadjust = obs.gs_effect_get_param_by_name(filter.effect, 'gadjust')
        filter.params.badjust = obs.gs_effect_get_param_by_name(filter.effect, 'badjust')
        filter.params.center = obs.gs_effect_get_param_by_name(filter.effect, 'center')
        filter.params.show = obs.gs_effect_get_param_by_name(filter.effect, 'show')
        filter.params.width = obs.gs_effect_get_param_by_name(filter.effect, 'width')
        filter.params.height = obs.gs_effect_get_param_by_name(filter.effect, 'height')
    end


    if filter.effect == nil then
		--shader = ''
        source_info.destroy(filter)
        return nil
    end

    source_info.update(filter, settings)
    return filter
end

source_info.destroy = function(filter)
    if filter.effect ~= nil then
        obs.obs_enter_graphics()
        obs.gs_effect_destroy(filter.effect)
        obs.obs_leave_graphics()
    end
	filter = nil
end

source_info.get_width = function(filter)
    return filter.width
end

source_info.get_height = function(filter)
    return filter.height
end

source_info.get_properties = function(settings)
    props = obs.obs_properties_create()

    obs.obs_properties_add_float_slider(props, SETTING_RAD, TEXT_RAD, 0.0, 0.1, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_GAD, TEXT_GAD, 0.0, 0.1, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_BAD, TEXT_BAD, 0.0, 0.1, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_CENT, TEXT_CENT, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_bool(props, SETTING_SHOW, TEXT_SHOW)

    return props
end

source_info.get_defaults = function(settings)
    obs.obs_data_set_default_double(settings, SETTING_RAD, 0.03)
    obs.obs_data_set_default_double(settings, SETTING_GAD, 0.02)
    obs.obs_data_set_default_double(settings, SETTING_BAD, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_CENT, 0.5)
    obs.obs_data_set_default_bool(settings, SETTING_SHOW, false)
end

source_info.update = function(filter, settings)
    filter.radjust = obs.obs_data_get_double(settings, SETTING_RAD)
    filter.gadjust = obs.obs_data_get_double(settings, SETTING_GAD)
    filter.badjust = obs.obs_data_get_double(settings, SETTING_BAD)
    filter.center = obs.obs_data_get_double(settings, SETTING_CENT)
    filter.show = obs.obs_data_get_bool(settings, SETTING_SHOW)

    set_render_size(filter)
end

source_info.video_render = function(filter)

  if not obs.obs_source_process_filter_begin(filter.source, obs.GS_RGBA, obs.OBS_ALLOW_DIRECT_RENDERING) then return end

    obs.gs_effect_set_float(filter.params.radjust, filter.radjust)
    obs.gs_effect_set_float(filter.params.gadjust, filter.gadjust)
    obs.gs_effect_set_float(filter.params.badjust, filter.badjust)
    obs.gs_effect_set_float(filter.params.center, filter.center)
    obs.gs_effect_set_bool(filter.params.show, filter.show)
	obs.gs_effect_set_int(filter.params.width, filter.width)
	obs.gs_effect_set_int(filter.params.height, filter.height)

	obs.gs_blend_state_push()
	obs.gs_blend_function(obs.GS_BLEND_ONE, obs.GS_BLEND_INVSRCALPHA)
    obs.obs_source_process_filter_end(filter.source, filter.effect, filter.width, filter.height)
	obs.gs_blend_state_pop()
end

source_info.video_tick = function(filter, seconds)
    set_render_size(filter)
end



shader = [[
//Chromatic Aberration effect filter by khaver

#define EMPTY 0.0.xxxx

uniform float4x4 ViewProj;
uniform texture2d image;

uniform float radjust;
uniform float gadjust;
uniform float badjust;
uniform float center;
uniform bool show;

uniform int width;
uniform int height;

sampler_state textureSampler {
    Filter    = Linear; //Anisotropy;
    AddressU  = Border;
    AddressV  = Border;
	BorderColor = 00000000;
};

struct VertData {
    float4 pos : POSITION;
    float2 uv  : TEXCOORD0;
};

VertData VSDefault(VertData v_in)
{
    VertData vert_out;
    vert_out.pos = mul(float4(v_in.pos.xyz, 1.0), ViewProj);
    vert_out.uv  = v_in.uv;
    return vert_out;
}

float4 ps_main(VertData v_in) : TARGET
{
	if (radjust == 0.0 && gadjust == 0.0 && badjust == 0.0 && !show) return image.Sample(textureSampler, v_in.uv);
	float _Aspect = float(width) / float(height);
	float prot = (sqrt((_Aspect * _Aspect) + 1.0) * center) * 0.5;
	float alpha = image.Sample(textureSampler, v_in.uv).a;
	float2 fromCenter = v_in.uv - float2(0.5, 0.5);
	fromCenter.x = fromCenter.x * _Aspect;
	float dist = length(fromCenter);
	float delt = (dist <= (prot)) ? 0.0 : sin(radians((dist - (prot)) * 90.0));
	float2 xy1 = v_in.uv - float2(0.5,0.5);
	float2 xy;
	float3 col;
	float2 RXYscale = float2((delt * radjust) + 1.0, (delt * radjust) + 1.0);
	xy = (xy1 / RXYscale) + float2(0.5,0.5);
	col.r = image.Sample(textureSampler, xy).r;
	float2 GXYscale = float2((delt * gadjust) + 1.0, (delt * gadjust) + 1.0);
	xy = (xy1 / GXYscale) + float2(0.5,0.5);
	col.g = image.Sample(textureSampler, xy).g;
	float2 BXYscale = float2((delt * badjust) + 1.0, (delt * badjust) + 1.0);
	xy = (xy1 / BXYscale) + float2(0.5,0.5);
	col.b = image.Sample(textureSampler, xy).b;
	if (show) if (dist > (prot) && (dist < (prot) + 0.0025)) col.rgb = float3(1.0, 0.0, 0.0);
	return float4(col, alpha);


}

technique Draw
{
    pass
    {
        vertex_shader = VSDefault(v_in);
        pixel_shader  = ps_main(v_in);
    }
}
]]